{% load i18n admin_urls %}

{% block object-tools-items %}

  {% if has_add_permission %}
    <li>
      {% url cl.opts|admin_urlname:'add' as add_url %}
      <a href="{% add_preserved_filters add_url is_popup to_field %}" class="addlink">
        {% blocktrans with cl.opts.verbose_name as name %}Add {{ name }}{% endblocktrans %}
      </a>
    </li>
  {% endif %}

  <li>
    <a href="remove-nodes/" class="">
      Remove Zwave Node <i class="fas fa-times"></i>
    </a>
  </li>
{% endblock %}
