# Generated by Django 3.2.9 on 2022-07-07 14:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('simo_zwave', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='nodevalue',
            name='genre',
            field=models.CharField(db_index=True, max_length=100, null=True),
        ),
    ]
